Package: wxgenR
Type: Package
Title: A Stochastic Weather Generator with Seasonality
Version: 1.4.5
Authors@R: 
  c(person(given = "Subhrendu",
           family = "Gangopadhyay",
           role = "aut"),
    person(given = "Lindsay",
           family = "Bearup",
           role = "aut"),
    person(given = "David",
           family = "Woodson",
           role = c("aut", "cre"),
           email = "dwoodson@usbr.gov"),  
    person(given = "Marketa",
           family = "McGuire",
           role = "aut"),
    person(given = "Andrew",
           family = "Verdin",
           role = "aut"), 
    person(given = "Eylon",
           family = "Shamir",
           role = "aut"),
    person(given = "Eve",
           family = "Halper",
           role = "aut"))         
Description: A weather generator to simulate precipitation and temperature for regions with seasonality. Users input training data containing precipitation, temperature, and seasonality (up to 26 seasons). Including weather season as a training variable allows users to explore the effects of potential changes in season duration as well as average start- and end-time dates due to phenomena like climate change. Data for training should be a single time series but can originate from station data, basin averages, grid cells, etc.
    Bearup, L., Gangopadhyay, S., & Mikkelson, K. (2021). "Hydroclimate Analysis Lower Santa Cruz River Basin Study (Technical Memorandum No ENV-2020-056)." Bureau of Reclamation.
    Gangopadhyay, S., Bearup, L. A., Verdin, A., Pruitt, T., Halper, E., & Shamir, E. (2019, December 1). "A collaborative stochastic weather generator for climate impacts assessment in the Lower Santa Cruz River Basin, Arizona." Fall Meeting 2019, American Geophysical Union. <https://ui.adsabs.harvard.edu/abs/2019AGUFMGC41G1267G>.
License: CC0
Copyright: This software is in the public domain because it contains
        materials that originally came from the United States Bureau of
        Reclamation, an agency of the United States Department of
        Interior.
Encoding: UTF-8
LazyData: true
LazyDataCompression: xz
Imports: lubridate, msm, sm, dplyr, plyr, foreach, parallel,
        doParallel, stats, utils, magrittr, doRNG, mc2d, qmap, tidyr
Suggests: knitr, rmarkdown, reshape2, ggpubr, moments, seas, padr,
        hydroGOF, SpecsVerification, ggridges, tigris, terra, stringr,
        mapview, xts, sf, lfstat, FedData, ggplot2
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-04 18:08:42 UTC; dwoodson
Depends: R (>= 3.5.0)
Config/roxygen2/version: 8.0.0
Author: Subhrendu Gangopadhyay [aut],
  Lindsay Bearup [aut],
  David Woodson [aut, cre],
  Marketa McGuire [aut],
  Andrew Verdin [aut],
  Eylon Shamir [aut],
  Eve Halper [aut]
Maintainer: David Woodson <dwoodson@usbr.gov>
Repository: CRAN
Date/Publication: 2026-05-04 19:30:02 UTC
Built: R 4.6.0; ; 2026-05-04 22:12:05 UTC; unix
