# Generated by roxygen2: do not edit by hand

export(generate_TmaxTmin)
export(getDatesInWindow)
export(multisite_shuffle)
export(repan)
export(selectState)
export(spellLengths)
export(writeSim)
export(wx)
import(doParallel)
import(doRNG)
import(foreach)
import(lubridate)
import(magrittr)
import(mc2d)
import(msm)
import(parallel)
import(qmap)
import(sm)
import(stats, except = filter)
import(utils)
importFrom(doRNG,registerDoRNG)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,glimpse)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,relocate)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(lubridate,leap_year)
importFrom(lubridate,ymd)
importFrom(magrittr,"%>%")
importFrom(mc2d,cornode)
importFrom(plyr,ddply)
importFrom(qmap,doQmapQUANT)
importFrom(qmap,fitQmapQUANT)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,starts_with)
