#' Example meteorological training data for weather generator
#'
#' Weather data (precipitation, temperature, and season) for the Lower Santa Cruz River Basin in Southern Arizona.
#' Dataset was developed for the Hydroclimate Analysis within Reclamation's Lower Santa Cruz River Basin Study.
#'
#'
#' @docType data
#'
#' @usage data(LowerSantaCruzRiverBasinAZ)
#'
#' @format A data frame
#'
#' @keywords data
#'
#' @source Hydroclimate Analysis - Lower Santa Cruz River Basin Study
#'
#' @examples
#' data(LowerSantaCruzRiverBasinAZ)
"LowerSantaCruzRiverBasinAZ"
