utils::globalVariables(c(
  "sim_temp", "sim_tmax", "sim_tmin", "simulation", "monthly_temp",
  "prcp", "sim_prcp", "station", "variable", "season", "month",
  "date", "year", "day", "temp", "internal_name", "value",
  "sim_variable", "Tag"
))
