#' Example meteorological training data for weather generator
#'
#' Weather data (precipitation, temperature, and season) measured at the NWS station
#' (GHCND:USC00440766) in Blacksburg, Virginia.
#'
#' @docType data
#'
#' @usage data(BlacksburgVA)
#'
#' @format A data frame.
#'
#' @keywords data
#'
#' @source \href{https://www.ncdc.noaa.gov/cdo-web/datasets/GHCND/stations/GHCND:USC00440766/detail}{Blacksburg, VA NWS office}
#'
#' @examples
#' data(BlacksburgVA)
"BlacksburgVA"
