#' wxgenR simulation results for multiple meteorological stations in Boulder County, CO
#'
#' Simulated precipitation and temperature at the following GHCN stations (data retrieved using the FedData R package):
#' "USC00050848" "USS0005J42S"
#'
#' @docType data
#'
#' @usage data(BOCO_sims)
#'
#' @format A list of dataframes.
#'
#' @keywords data
#'
#' @source \href{https://www.ncei.noaa.gov/products/land-based-station/global-historical-climatology-network-daily}{GHCNd}
#'
#' @examples
#' data(BOCO_sims)
"BOCO_sims"
